#include <JUI/Base/RectBase.hpp>
#include <JGL/JGL.h>

using namespace JGL;

namespace JUI {
    RectBase::RectBase() = default;

    bool RectBase::GetClipsDescendants() const { return clips_descendants;}

    void RectBase::SetClipsDescendants(bool clipping) { clips_descendants = clipping; }

    void RectBase::BGColor(const Color4 &col) { bg_color = col; }

    void RectBase::BorderColor(const Color4 &col) { border_color = col; }

    void RectBase::BorderWidth(float w) { border_width = w; }

    float RectBase::GetBorderWidth() const { return border_width; }

    Color4 RectBase::GetBorderColor() const { return border_color; }

    Color4 RectBase::BGColor() const { return bg_color; }

    void RectBase::SetBorderStyling(const Color4 &color, float width) {
        BorderColor(color);
        BorderWidth(width);
    }


    void RectBase::Draw(const Vector2& abs_pos, const Vector2& abs_size) {
        Draw(bg_color, border_color, abs_pos, abs_size);
    }

    void RectBase::Draw(const Color4& bgColor, const Color4& borderColor, const Vector2 &abs_pos, const Vector2 &abs_size) {

        // Background rect
        if (corner_rounding_radius > 0)
            J2D::FillRoundedRect(bgColor, abs_pos, abs_size, corner_rounding_radius);
        else
            J2D::FillRect(bgColor, abs_pos, abs_size);

        // Outline rect - compute the size change to fit the border accurately.
        Vector2 border_offset = {0, 0};

        if (border_mode == BorderMode::Inset)
            border_offset = {-border_width/2.f, -border_width/2.f};
        if (border_mode == BorderMode::Middle)
            border_offset =  {0, 0};
        if (border_mode == BorderMode::Outline)
            border_offset = {border_width/2.f, border_width/2.f};

        // Draw the outline.
        if (border_width > 0) {
            if (corner_rounding_radius > 0)
                J2D::OutlineRoundedRect(borderColor, abs_pos - border_offset, abs_size + (border_offset*2), corner_rounding_radius, border_width);
            else
                J2D::OutlineRect(borderColor, abs_pos - border_offset, abs_size + (border_offset*2), border_width);
        }

    }

    void RectBase::BorderMode(const enum BorderMode &mode) {
        border_mode = mode;
    }

    BorderMode RectBase::BorderMode() const { return border_mode; }

    void RectBase::CornerRounding(float radius) {
        corner_rounding_radius = radius;
    }

    float RectBase::CornerRounding() const { return corner_rounding_radius; }


}