#include <JUI/Widgets/Separator.hpp>


namespace JUI {
    Separator::Separator(): Widget() {
        Name("Separator");
    }

    Separator::Separator(Widget *parent): Separator() {
        Parent(parent);
    }

    void Separator::Orient(const enum JUI::Orientation &orientation) {
        this->orientation = orientation;
    }

    void Separator::LineFillMode(const enum JUI::LineFillMode &line_mode) {
        this->line_mode = line_mode;
    }

    Color4 Separator::LineColor() const { return line_color; }

    void Separator::LineColor(const Color4 &color) {
        line_color = color;
    }

    void Separator::InnerDraw() {

        Vector2 abs_pos = GetAbsolutePosition();
        Vector2 abs_size = GetAbsoluteSize();

        Vector2 start = abs_pos;
        Vector2 end = abs_size;

        // TODO: Factor in padding.

        if (orientation == Orientation::Horizontal) {
            start = abs_pos + Vector2(0, abs_size.y / 2.f);
            end = start + Vector2(abs_size.x, 0);
        }

        if (orientation == Orientation::Vertical) {
            start = abs_pos + Vector2(abs_size.x / 2.f, 0);
            end = start + Vector2(0, abs_size.y);
        }

        if (line_mode == LineFillMode::Dotted)
            J2D::DrawDottedLine(line_color, start, end, spacing, line_thickness);

        if (line_mode == LineFillMode::Dashed)
            J2D::DrawDashedLine(line_color, start, end ,spacing, line_thickness);

        if (line_mode == LineFillMode::Solid)
            JGL::J2D::DrawLine(line_color, start, end, line_thickness);
    }

    float Separator::Spacing() const { return spacing; }

    void Separator::Spacing(float spacing) {
        this->spacing = spacing;
    }

    enum Orientation Separator::Orientation() const { return orientation;}

    enum LineFillMode Separator::LineFillMode() const { return line_mode;}
}